@@ -1,6 +1,5 @@
 import io
 import sys
-import unittest
 import unittest.mock as mock
 from contextlib import contextmanager
 from functools import partial
@@ -32,14 +31,14 @@ def captured_output():
         sys.stdout, sys.stderr = old_out, old_err
 
 
-class TestInterface(unittest.TestCase):
+class TestInterface:
     def test_close(self):
         io = Interface(lambda input: None, "textbox", "label")
         _, local_url, _ = io.launch(prevent_thread_lock=True)
         response = requests.get(local_url)
-        self.assertEqual(response.status_code, 200)
+        assert response.status_code == 200
         io.close()
-        with self.assertRaises(Exception):
+        with pytest.raises(Exception):
             response = requests.get(local_url)
 
     def test_close_all(self):
@@ -49,7 +48,7 @@ class TestInterface(unittest.TestCase):
         interface.close.assert_called()
 
     def test_no_input_or_output(self):
-        with self.assertRaises(TypeError):
+        with pytest.raises(TypeError):
             Interface(lambda x: x, examples=1234)
 
     def test_partial_functions(self):
@@ -68,27 +67,18 @@ class TestInterface(unittest.TestCase):
         dataset_check = any(
             [c["type"] == "dataset" for c in interface.get_config_file()["components"]]
         )
-        self.assertTrue(dataset_check)
-
-    def test_test_launch(self):
-        with captured_output() as (out, err):
-            prediction_fn = lambda x: x
-            prediction_fn.__name__ = "prediction_fn"
-            interface = Interface(prediction_fn, "textbox", "label")
-            interface.test_launch()
-            output = out.getvalue().strip()
-            self.assertEqual(output, "Test launch: prediction_fn()... PASSED")
+        assert dataset_check
 
     @mock.patch("time.sleep")
     def test_block_thread(self, mock_sleep):
-        with self.assertRaises(KeyboardInterrupt):
+        with pytest.raises(KeyboardInterrupt):
             with captured_output() as (out, _):
                 mock_sleep.side_effect = KeyboardInterrupt()
                 interface = Interface(lambda x: x, "textbox", "label")
                 interface.launch(prevent_thread_lock=False)
                 output = out.getvalue().strip()
-                self.assertEqual(
-                    output, "Keyboard interruption in main thread... closing server."
+                assert (
+                    output == "Keyboard interruption in main thread... closing server."
                 )
 
     @mock.patch("gradio.utils.colab_check")
@@ -96,7 +86,7 @@ class TestInterface(unittest.TestCase):
         mock_colab_check.return_value = True
         interface = Interface(lambda x: x, "textbox", "label")
         _, _, share_url = interface.launch(prevent_thread_lock=True)
-        self.assertIsNotNone(share_url)
+        assert share_url is None
         interface.close()
 
     @mock.patch("gradio.utils.colab_check")
@@ -106,15 +96,15 @@ class TestInterface(unittest.TestCase):
         mock_colab_check.return_value = True
         interface = Interface(lambda x: x, "textbox", "label")
         _, _, share_url = interface.launch(prevent_thread_lock=True)
-        self.assertIsNone(share_url)
+        assert share_url is None
         interface.close()
 
     def test_interface_representation(self):
         prediction_fn = lambda x: x
         prediction_fn.__name__ = "prediction_fn"
         repr = str(Interface(prediction_fn, "textbox", "label")).split("\n")
-        self.assertTrue(prediction_fn.__name__ in repr[0])
-        self.assertEqual(len(repr[0]), len(repr[1]))
+        assert prediction_fn.__name__ in repr[0]
+        assert len(repr[0]) == len(repr[1])
 
     @pytest.mark.asyncio
     async def test_interface_none_interp(self):
@@ -122,7 +112,7 @@ class TestInterface(unittest.TestCase):
         scores = (await interface.interpret(["quickest brown fox"]))[0][
             "interpretation"
         ]
-        self.assertIsNone(scores)
+        assert scores is None
 
     @mock.patch("webbrowser.open")
     def test_interface_browser(self, mock_browser):
@@ -133,10 +123,13 @@ class TestInterface(unittest.TestCase):
 
     def test_examples_list(self):
         examples = ["test1", "test2"]
-        interface = Interface(lambda x: x, "textbox", "label", examples=examples)
+        interface = Interface(
+            lambda x: x, "textbox", "label", examples=examples, examples_per_page=2
+        )
         interface.launch(prevent_thread_lock=True)
-        self.assertEqual(len(interface.examples_handler.examples), 2)
-        self.assertEqual(len(interface.examples_handler.examples[0]), 1)
+        assert len(interface.examples_handler.examples) == 2
+        assert len(interface.examples_handler.examples[0]) == 1
+        assert interface.examples_handler.dataset.get_config()["samples_per_page"] == 2
         interface.close()
 
     @mock.patch("IPython.display.display")
@@ -145,7 +138,7 @@ class TestInterface(unittest.TestCase):
         interface.launch(inline=True, prevent_thread_lock=True)
         mock_display.assert_called_once()
         interface.launch(inline=True, prevent_thread_lock=True)
-        self.assertEqual(mock_display.call_count, 2)
+        assert mock_display.call_count == 2
         interface.close()
 
     @mock.patch("comet_ml.Experiment")
@@ -160,7 +153,7 @@ class TestInterface(unittest.TestCase):
         interface.share_url = "tmp"  # used to avoid creating real share links.
         interface.integrate(comet_ml=experiment)
         experiment.log_text.assert_called_with("gradio: " + interface.share_url)
-        self.assertEqual(experiment.log_other.call_count, 2)
+        assert experiment.log_other.call_count == 2
         interface.share_url = None
         interface.close()
 
@@ -189,9 +182,9 @@ class TestInterface(unittest.TestCase):
             interface.height = 500
             interface.integrate(wandb=wandb)
 
-            self.assertEqual(
-                out.getvalue().strip(),
-                "The WandB integration requires you to `launch(share=True)` first.",
+            assert (
+                out.getvalue().strip()
+                == "The WandB integration requires you to `launch(share=True)` first."
             )
             interface.share_url = "tmp"
             interface.integrate(wandb=wandb)
@@ -206,7 +199,7 @@ class TestInterface(unittest.TestCase):
         mock_post.assert_called_once()
 
 
-class TestTabbedInterface(unittest.TestCase):
+class TestTabbedInterface:
     def test_tabbed_interface_config_matches_manual_tab(self):
         interface1 = Interface(lambda x: x, "textbox", "textbox")
         interface2 = Interface(lambda x: x, "image", "image")
@@ -222,20 +215,18 @@ class TestTabbedInterface(unittest.TestCase):
         interface4 = Interface(lambda x: x, "image", "image")
         tabbed_interface = TabbedInterface([interface3, interface4], ["tab1", "tab2"])
 
-        self.assertTrue(
-            assert_configs_are_equivalent_besides_ids(
-                demo.get_config_file(), tabbed_interface.get_config_file()
-            )
+        assert assert_configs_are_equivalent_besides_ids(
+            demo.get_config_file(), tabbed_interface.get_config_file()
         )
 
 
-class TestDeprecatedInterface(unittest.TestCase):
+class TestDeprecatedInterface:
     def test_deprecation_notice(self):
-        with self.assertWarns(Warning):
+        with pytest.warns(Warning):
             _ = Interface(lambda x: x, "textbox", "textbox", verbose=True)
 
 
-class TestInterfaceInterpretation(unittest.TestCase):
+class TestInterfaceInterpretation:
     def test_interpretation_from_interface(self):
         def quadratic(num1: float, num2: float) -> float:
             return 3 * num1**2 + num2
@@ -247,6 +238,34 @@ class TestInterfaceInterpretation(unittest.TestCase):
             interpretation="default",
         )
 
+        interpretation_id = None
+        for c in iface.config["components"]:
+            if c["props"].get("value") == "Interpret" and c.get("type") == "button":
+                interpretation_id = c["id"]
+
+        # Make sure the event is configured correctly.
+        interpretation_dep = next(
+            d
+            for d in iface.config["dependencies"]
+            if d["targets"] == [interpretation_id]
+        )
+        interpretation_comps = [
+            c["id"]
+            for c in iface.config["components"]
+            if c.get("type") == "interpretation"
+        ]
+        interpretation_columns = [
+            c["id"]
+            for c in iface.config["components"]
+            if c.get("type") == "column" and c["props"].get("variant") == "default"
+        ]
+        assert sorted(interpretation_dep["outputs"]) == sorted(
+            interpretation_comps + interpretation_columns
+        )
+        assert sorted(interpretation_dep["inputs"]) == sorted(
+            [c._id for c in iface.input_components + iface.output_components]
+        )
+
         app, _, _ = iface.launch(prevent_thread_lock=True)
         client = TestClient(app)
 
@@ -264,7 +283,7 @@ class TestInterfaceInterpretation(unittest.TestCase):
         response = client.post(
             "/api/predict/", json={"fn_index": fn_index, "data": [10, 50, 350]}
         )
-        self.assertTrue(response.json()["data"][0]["interpretation"] is not None)
+        assert response.json()["data"][0]["interpretation"] is not None
         iface.close()
         close_all()
 
@@ -312,7 +331,3 @@ def test_interface_adds_stop_button(interface_type, live, use_generator):
         assert has_stop
     else:
         assert not has_stop
-
-
-if __name__ == "__main__":
-    unittest.main()
